/*
    Copyright (C) 2002, Steve Lhomme <steve.lhomme @ free.fr>
    This file is part of mpa2wav.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/**
    \file mpa2wav.cpp
    Convert MPEG I/II Layer 1/2/3 to RIFF/WAV/MPEG files
    \todo remove bad frames
    \todo the ID3v1 and v2 data could be put in a special WAV chunk
*/
#include <stdio.h>
#include <string>
#include <cassert>

#include "mpafile.h"
#include "mpiframe.h"
#include "mpaframe.h"
#include "mpg_wav.h"

#if !defined(NO_NAMESPACE)
using namespace std;
using namespace M2W_NAMESPACE;
#endif

const char version[] = "0.2.2";

void usage()
{
    printf("This program converts MPEG I/II Layer 1/2/3 to RIFF-WAV-MPEG files\r\n\r\n");
    printf("Usage: mpa2wav <input-filename> [output-filename]\r\n");
//    printf("Usage: mpa2wav [-v] <input-filename> [output-filename]\r\n");
//    printf("\t-v verbose\n");
    printf("\tif no output file is specified the input filename is appended with .wav\r\n");
}

int main(int argc, char*argv[])
{
    bool bVerbose = true;
    bool bVerboseMore = false;

    printf("mpa2wav v%s - (c) 2002 Steve Lhomme - GPL License\r\n", version);
    printf("Some code was borrowed from libMCF (c) 2002 - LGPL License\r\n");
    // check for the valid number of parameters
    if (argc != 2 && argc != 3)
    {
	usage();
	return -1;
    }

    string input_Filename = argv[1];

    string output_Filename;
    if (argc == 3)
	output_Filename = argv[2];
    else
	output_Filename = input_Filename + ".wav";

    if (bVerbose)
	printf("Converting %s to %s\r\n", input_Filename.c_str(), output_Filename.c_str());

    // open the input file
    MPA_File input(input_Filename, mode_read);

    // open the output file
    MPG_WAV_file output(output_Filename);
    if (!output.Open(FILE_MODE_WRITE | FILE_MODE_ERASE))
    {
	printf("Impossible to open output file '%s'\r\n", output_Filename.c_str());
	return -1;
    }

    // prepare to read the whole file for statistics
    if (bVerbose)
	printf("Reading statistics in %s\r\n", input_Filename.c_str());

//    Mpeg_Layer layer = MPEG_LAYER_UNKNOWN;
    input.SkipToFrameStart();

    bool bInfoFrameOn = false;
//    if (layer == MPEG_LAYER_3)
    {
	MPA_Lame_Frame infoFrame;
	if (input.CheckInfoFrame())
	{
	    *static_cast<MPA_Frame *>(&infoFrame) = input.GetInfoFrame();
	    bInfoFrameOn = true;
	}
    }

    uint32 FrameNumber = 0;
    uint64 SampleNumbers = 0;
    uint64 TotalFrameSize = 0;
//    uint16 SamplingRate = 0;
//    uint16 Channels = 0;
    uint16 SampleSize = 0;
    MPA_Frame frame;

    while (input.FindNextSyncWord())
    {
	FrameNumber++;
	frame = input.GetFrame();
	if (output.Feed(frame))
	{
	    /// \todo handle the bitrate of the frame in a key/value table, so that we can detect real CBR files
	    TotalFrameSize += frame.DataSize() + 4;
	    SampleNumbers += frame.SampleSize();
//	    SamplingRate = frame.SamplingRate();
//	    Channels = frame.Channels();
	    SampleSize = frame.SampleSize();
	}

	input.SkipData();
    }
    output.FixStats();

    uint64 totalByteSize = TotalFrameSize * output.SamplingRate();
    uint32 AverageBitrate, AverageByterate;
    if ((8 * totalByteSize) % SampleNumbers > SampleNumbers>>1)
	AverageBitrate = 1 + 8 * totalByteSize / SampleNumbers;
    else
	AverageBitrate = 8 * totalByteSize / SampleNumbers;

    if (AverageBitrate % 8 > 4)
	AverageByterate = 1 + (AverageBitrate >> 3);
    else
	AverageByterate = AverageBitrate >> 3;

    if (bVerboseMore)
    {
	printf("Sampling Rate = %d / Total Valid Frame Size = %ld\r\n",
	       output.SamplingRate(), TotalFrameSize);
	printf("Average Bitrate = %d (8 x %d)\r\n", AverageBitrate, AverageByterate);
	printf("Sample number = %d\r\n", SampleNumbers);
	printf("FrameNumber = %d\r\n", FrameNumber);
    }

    // construct the WAV header with the statistics

    // prepare to read for output
    input.SkipToFrameStart();

    /// \todo add the option to skip the Info frame or not
    if (bInfoFrameOn)
    {
	input.FindNextSyncWord();
	input.GetFrame();
	input.SkipData();
    }

    if (bVerbose)
	printf("Writing frames in %s\r\n", output_Filename.c_str());

    /// \todo move this somewhere else (cleaner)
    output.setByterate(AverageByterate);
    output.setSampleSize(SampleSize);

    output.WriteHead(TotalFrameSize, SampleNumbers, false);

    binary frame_buffer[5000]; // should be enough for 32kHz/320kbps
    while (input.FindNextSyncWord())
    {
	frame = input.GetFrame();
	if (output.Test(frame))
	{
	    output.Write(frame, 4);
	    assert(frame.DataSize() < 5000);
	    input.GetData(frame_buffer, 5000);
	    output.Write(frame_buffer, frame.DataSize());
	}
	else input.SkipData();
    }

    output.Close();

    return 0;
}
