/*
    Copyright (C) 2002, Steve Lhomme <steve.lhomme @ free.fr>
    This file is part of mpa2wav.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/**
    \file
*/

#ifndef _M2W_MPG_FORMAT_H_
#define _M2W_MPG_FORMAT_H_

#include "format.h"
#include "mpaframe.h"

START_M2W_NAMESPACE

/**
    \class MPG_FORMAT_chunk generic MPG chunk class
*/
class MPG_FORMAT_chunk : public FORMAT_chunk {
    public:
	MPG_FORMAT_chunk()
	    :mySampleSize(0)
	    ,myLayer(MPEG_LAYER_UNKNOWN)
	    ,myBitRate(0)
	    ,myCBR(false)
	    ,myEmphasis(MPEG_EMPH_NONE)
	    ,myPrivate(false)
	    ,myCopyright(false)
	    ,myOriginal(false)
	    ,myProtection(false)
	{}

	virtual void setSampleSize(const uint16 size) {mySampleSize = size;}
	void setLayer(const Mpeg_Layer aLayer) {myLayer = aLayer;}
	void setChannelMode(const Mpeg_Channel value) {myChannels = value;}
	void setBitrate(const uint32 value, const bool CBR)
	{
	    myBitRate = value;
	    myCBR = CBR;
	}
	void setEmphasis(const Mpeg_Emphasis value) {myEmphasis = value;}
	void setFlags(const bool priv, const bool copy, const bool orig, const bool protec)
	{
	    myPrivate = priv;
	    myCopyright = copy;
	    myOriginal = orig;
	    myProtection = protec;
	}

	const Mpeg_Layer Layer() const {return myLayer;}
	const Mpeg_Channel ChannelMode() const {return myChannels;}
	const Mpeg_Emphasis Emphasis() const {return myEmphasis;}

	virtual uint32 Write(IO_file & aFile) = 0;
	virtual uint16 FormatTag() const = 0;
	virtual uint16 FormatTagSize() const = 0;

    protected:
	uint16        mySampleSize;
	Mpeg_Layer    myLayer;
	Mpeg_Channel  myChannels;
	uint32        myBitRate;
	bool          myCBR;
	Mpeg_Emphasis myEmphasis;
	bool          myPrivate;
	bool          myCopyright;
	bool          myOriginal;
	bool          myProtection;
};

const uint16 WAVE_FORMAT_MPEGLAYER3 = 0x0055;
const uint8 MPEGLAYER3_WFX_EXTRA_BYTES = 12; // due to alignment problems

typedef struct {
  uint16 wID;
  uint32 fdwFlags;
  uint16 nBlockSize;
  uint16 nFramesPerBlock;
  uint16 nCodecDelay;
} MPEGLAYER3WAVEFORMATEX;

/// \todo use these defines
// for wID
const uint16 MPEGLAYER3_ID_UNKNOWN           = 0;
const uint16 MPEGLAYER3_ID_MPEG              = 1;
const uint16 MPEGLAYER3_ID_CONSTANTFRAMESIZE = 2;
// for fdwFlags
const uint32 MPEGLAYER3_FLAG_PADDING_ISO = 0x00000000;
const uint32 MPEGLAYER3_FLAG_PADDING_ON  = 0x00000001;
const uint32 MPEGLAYER3_FLAG_PADDING_OFF = 0x00000002;

/**
    \class MP3_FORMAT_chunk MPEG Layer 3 specific class
*/
class MP3_FORMAT_chunk : public MPG_FORMAT_chunk {
    public:
	MP3_FORMAT_chunk()
	{
	    myFormatEx.wID = MPEGLAYER3_ID_MPEG;
	    myFormatEx.fdwFlags = MPEGLAYER3_FLAG_PADDING_OFF;
	    myFormatEx.nCodecDelay = 0;
	    myFormatEx.nFramesPerBlock = 1;
	}

	virtual uint32 Write(IO_file & aFile);
	virtual uint16 FormatTag() const {return WAVE_FORMAT_MPEGLAYER3;}
	virtual uint16 FormatTagSize() const {return MPEGLAYER3_WFX_EXTRA_BYTES;}

    protected:
	MPEGLAYER3WAVEFORMATEX myFormatEx;
};

const uint16 WAVE_FORMAT_MPEG = 0x0050;
const uint8 MPEG1_WFX_EXTRA_BYTES = 22; // due to alignment problems

typedef struct {
  uint16 fwHeadLayer;
  uint32 dwHeadBitrate;
  uint16 fwHeadMode;
  uint16 fwHeadModeExt;
  uint16 wHeadEmphasis;
  uint16 fwHeadFlags;
  uint32 dwPTSLow;
  uint32 dwPTSHigh;
} MPEG1WAVEFORMATEX;

// for fwHeadLayer
const uint16 ACM_MPEG_LAYER1         = 0x0001;
const uint16 ACM_MPEG_LAYER2         = 0x0002;
const uint16 ACM_MPEG_LAYER3         = 0x0004;
// for fwHeadMode
const uint16 ACM_MPEG_STEREO         = 0x0001;
const uint16 ACM_MPEG_JOINTSTEREO    = 0x0002;
const uint16 ACM_MPEG_DUALCHANNEL    = 0x0004;
const uint16 ACM_MPEG_SINGLECHANNEL  = 0x0008;
// for fwHeadFlags
const uint16 ACM_MPEG_PRIVATEBIT     = 0x0001;
const uint16 ACM_MPEG_COPYRIGHT      = 0x0002;
const uint16 ACM_MPEG_ORIGINALHOME   = 0x0004;
const uint16 ACM_MPEG_PROTECTIONBIT  = 0x0008;
const uint16 ACM_MPEG_ID_MPEG1       = 0x0010;
// for fwHeadEmphasis
const uint16 ACM_MPEG_NO_EMPHASIS    = 0x0001;
const uint16 ACM_MPEG_50_15MS        = 0x0002;
const uint16 ACM_MPEG_RESERVED       = 0x0004;
const uint16 ACM_MPEG_CCITT_J17      = 0x0008;

/**
    \class MP3_FORMAT_chunk MPEG 1 specific class
*/
class MPEG1_FORMAT_chunk : public MPG_FORMAT_chunk {
    public:
	MPEG1_FORMAT_chunk()
	{
	    myFormatEx.fwHeadModeExt = 0;
	    myFormatEx.dwPTSLow = 0;
	    myFormatEx.dwPTSHigh = 0;
	}

	virtual void setSampleSize(const uint16 size)
	{
/** \todo might work for smaller sampling rate too
	    if (mySampleSize != 44100 && 
		mySampleSize != 32000 && 
		mySampleSize != 48000)
		throw 0;
*/
	    MPG_FORMAT_chunk::setSampleSize(size);
	}
	
	virtual uint32 Write(IO_file & aFile);
	virtual uint16 FormatTag() const {return WAVE_FORMAT_MPEG;}
	virtual uint16 FormatTagSize() const {return MPEG1_WFX_EXTRA_BYTES;}

    protected:
	MPEG1WAVEFORMATEX myFormatEx;
};

END_M2W_NAMESPACE

#endif // _M2W_FORMAT_H_
